/*
 *  GRUB  --  GRand Unified Bootloader
 *  Copyright (C) 2003,2005,2006,2007,2009  Free Software Foundation, Inc.
 *
 *  GRUB is free software: you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation, either version 3 of the License, or
 *  (at your option) any later version.
 *
 *  GRUB is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with GRUB.  If not, see <http://www.gnu.org/licenses/>.
 */

#include <grub/dl.h>
#include <grub/err.h>
#include <grub/command.h>
#include <grub/device.h>
#include <grub/disk.h>
#include <grub/env.h>

GRUB_MOD_LICENSE ("GPLv3+");

#define FIRM_MAGIC_STR "xds."
#define FIRM_MAGIC_LEN 4

static grub_err_t
set_recovery (void)
{
	const char *recovery_root = grub_env_get("recovery_root");
	const char *recovery_configfile = grub_env_get("recovery_configfile");
	grub_command_t cfg_cmd;
	char *cfg_args[1];

	if (!recovery_root) {
		grub_printf ("recovery_root is not set.\n");
		return 0;
	}

	grub_env_set("root", recovery_root);

	if (!recovery_configfile) {
		return 0;
	}

	cfg_args[0] = (char *)recovery_configfile;

	cfg_cmd = grub_command_find ("configfile");
	if (cfg_cmd) {
		return (cfg_cmd->func) (cfg_cmd, sizeof(cfg_args)/sizeof(cfg_args[0]), cfg_args);
	}

	return 0;
}

static grub_err_t
do_check_normal (grub_command_t cmd __attribute__ ((unused)),
				 int argc,
				 char *args[])
{
	grub_device_t dev;
	grub_off_t offset;
	grub_err_t err = 0;
	char magic[4];

	if (argc != 2) {
		return grub_error (GRUB_ERR_BAD_ARGUMENT, N_("two arguments expected"));
	}

	offset = grub_strtoull (args[1], NULL, 16);

	dev = grub_device_open (args[0]);
	if (!dev) {
		return grub_error (GRUB_ERR_BAD_DEVICE, N_("can't open device"));
	}

	if (!dev->disk) {
		grub_printf ("dev->disk is NULL\n");
		err = GRUB_ERR_BAD_DEVICE;
		goto out_close;
	}

	err = grub_disk_read (dev->disk,
						  offset / GRUB_DISK_SECTOR_SIZE,
						  offset % GRUB_DISK_SECTOR_SIZE,
						  sizeof(magic),
						  magic);

	if (err) {
		grub_printf ("grub_disk_read() error(%d).\n", err);
		err = GRUB_ERR_READ_ERROR;
		goto out_close;
	}

	grub_printf ("magic: %02x %02x %02x %02x\n",
				 magic[0],
				 magic[1],
				 magic[2],
				 magic[3]);

	if (!grub_memcmp (magic, FIRM_MAGIC_STR, sizeof(magic))) {
		err = 0;
	} else {
		err = GRUB_ERR_BAD_FS;
	}

 out_close:
	return err;
}

static grub_err_t
grub_cmd_check_normal (grub_command_t cmd,
					   int argc,
					   char *args[])
{
	grub_err_t err;

	err = do_check_normal(cmd, argc, args);
	if (err) {
		return set_recovery();
	}

	return 0;
}

static grub_command_t cmd_check_normal;

GRUB_MOD_INIT(check_normal)
{
  cmd_check_normal =
    grub_register_command ("check_normal", grub_cmd_check_normal,
			   N_("FILE"), N_("check normal firmware."));
}

GRUB_MOD_FINI(check_normal)
{
  grub_unregister_command (cmd_check_normal);
}

/* end of check_normal.c */
